@@ -32,11 +32,16 @@
 
 #include "env.h"
 #include "lib/platform/util/edid.h"
+#include "lib/platform/X11/randr-edid.h"
 
 using namespace PLATFORM;
 
 uint16_t CEDIDParser::GetPhysicalAddress(void)
 {
+#if HAVE_RANDR
+  return CRandrEdidParser().GetPhysicalAddress();    
+#else
   // TODO
   return 0;
+#endif
 }
